%% Model Definition :: Human Body

global parts_name parts_np parts_nv parts_nvtot parts_nvcum 
global parts_nc parts_nctot parts_nccum parts_curv parts_clr
global parts_poly parts_ref
global links_name links_nl
global links_parts links_vtxa links_vtxb
global links_clr
global default_width default_height ivpal ippal

global parts_np_up parts_np_low parts_ip_up parts_ip_low parts_iv_up parts_iv_low
global parts_np_torso parts_np_arm parts_ip_torso parts_ip_arm parts_iv_torso parts_iv_arm
global parts_np_arml parts_np_armr parts_ip_arml parts_ip_armr parts_iv_arml parts_iv_armr
global parts_np_legl parts_np_legr parts_ip_legl parts_ip_legr parts_iv_legl parts_iv_legr
global links_nl_up links_nl_low links_il_up links_il_low

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% parts
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

parts_name = { 'head', 'torso', ...
               'legul', 'legur', 'legll', 'leglr', ...
               'armul', 'armur', 'armll', 'armlr', ...
               'handl', 'handr', 'footl', 'footr'  };
parts_np = length(parts_name);

% number of vertices
parts_nv = [ 6 14 ...
             8 8 10 10 ...
             4 4 4 4 ...
             6 6 4 4 ];
parts_nvtot = sum(parts_nv);
parts_nvcum = [0 cumsum(parts_nv)];

% number of curves
parts_nc = [ 1 2 ...
             2 2 2 2 ...
             2 2 2 2 ...
             1 1 1 1 ];
parts_nctot = sum(parts_nc);
parts_nccum = [0 cumsum(parts_nc)];

parts_curv = { [1 2 3 4 5],[1:2:13],[2:2:14], ...
               [1:2:7],[2:2:8],[1:2:7],[2:2:8],[1:2:9],[2:2:10],[1:2:9],[2:2:10], ...
               [1 3],[2 4],[1 3],[2 4],[1 3],[2 4],[1 3],[2 4], ...
               [1 3 5 6 4 2],[1 3 5 6 4 2],[1 3 4 2],[1 3 4 2] };
parts_clr = {'g','r','m','c','m','c','m','c','m','c','m','c','m','c'};

parts_poly = { [1:6 1],[1:2:13 14:-2:2 1], ...
               [1:2:7 8:-2:2 1],[1:2:7 8:-2:2 1],[1:2:9 10:-2:2 1],[1:2:9 10:-2:2 1], ...
               [1 3 4 2 1],[1 3 4 2 1],[1 3 4 2 1],[1 3 4 2 1], ...
               [1 3 5 6 4 2 1],[1 3 5 6 4 2 1],[1 3 4 2 1],[1 3 4 2 1] };

% reference line segment for procrustes anaysis
parts_ref = { [1 5],[13 14], ...
              [7 8],[7 8],[9 10],[9 10], ...
              [3 4],[3 4],[3 4],[3 4], ...
              [3 4],[3 4],[3 4],[3 4] };

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% joints/links
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

links_name = { 'neck', ...
               'hipl', 'hipr', 'keel', 'keer', 'ankl', 'ankr', ...
               'shl', 'shr', 'elbl', 'elbr', 'wristl', 'wristr' };
links_nl = length(links_name);

links_parts = [ 1 2; ...
                2 3; 2 4; 3 5; 4 6; 5 13; 6 14; ...
                2 7; 2 8; 7 9; 8 10; 9 11; 10 12 ];
links_vtxa =  [ 1 5; ...
                13 14; 13 14; 7 8; 7 8; 9 10; 9 10; ...
                3 4; 3 4; 3 4; 3 4; 3 4; 3 4 ];
links_vtxb = [ 3 4; ones(12,1)*[1 2] ];

links_clr = {'y','y','y','y','y','y','y','y','y','y','y','y','y'};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DP model & KP model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

parts_kp = { [1 2 6 3 5 4],[3 4 7 8 13 14], ...
             [1 2 7 8],[1 2 7 8],[1 2 9 10],[1 2 9 10], ...
             [1 2 3 4],[1 2 3 4],[1 2 3 4],[1 2 3 4], ...
             [1 2 5 6],[1 2 5 6],[1 2 3 4],[1 2 3 4] };
parts_nkp = [ 6 6 ...
              4 4 4 4 ...
              4 4 4 4 ...
              4 4 4 4 ];
parts_nkptot = sum(parts_nkp);
parts_nkpcum = [0 cumsum(parts_nkp)];

parts_order = { [1 2 6 3 5 4],[3 4 7 8 13 14 1 2 11 12 5 6 9 10], ...
             [1 2 7 8 5 6 3 4],[1 2 7 8 5 6 3 4],[1 2 9 10 5 6 3 4 7 8],[1 2 9 10 5 6 3 4 7 8], ...
             [1 2 3 4],[1 2 3 4],[1 2 3 4],[1 2 3 4], ...
             [1 2 5 6 3 4],[1 2 5 6 3 4],[1 2 3 4],[1 2 3 4] };
           
% number of DP steps
dp_ns = 18;

% connectivity graph
cueEdge = bitshift(1,0);
cueSim = bitshift(1,1);
cueSkin = bitshift(1,2);
cueFg = bitshift(1,3);
cueJoint = bitshift(1,8);
cueSkina = bitshift(1,9);
cueSkinb = bitshift(1,10);
cueSkino = bitshift(1,11);

j = cueJoint;
e = cueEdge;
u = bitor(bitor(cueEdge,cueSkin),cueSkina);
v = bitor(bitor(cueEdge,cueSkin),cueSkinb);
w = bitor(bitor(bitor(cueEdge,cueSkin),cueSkinb),cueSkino);

dp_graph = [ 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0;  % 1,
             0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0;  % 2, foot
             0 0 e 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0;  % 3,
             0 0 e 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0;  % 4, leg (lower)
             0 0 0 j 0 0 0 0 0 0 0 0 0 0 0 0 0 0;  % 5,
             0 0 0 0 e 0 0 0 0 0 0 0 0 0 0 0 0 0;  % 6, leg (upper)
             0 0 0 0 0 0 e 0 0 0 0 0 0 0 0 0 0 0;  % 7,
             0 0 0 0 0 0 v 0 0 0 0 0 0 0 0 0 0 0;  % 8, hand
             0 0 0 0 0 0 0 j 0 0 0 0 0 0 0 0 0 0;  % 9,
             0 0 0 0 0 0 0 0 w 0 0 0 0 0 0 0 0 0;  % 10, arm (lower)
             0 0 0 0 0 0 0 0 0 j 0 0 0 0 0 0 0 0;  % 11,
             0 0 0 0 0 0 0 0 0 0 e 0 0 0 0 0 0 0;  % 12, arm (upper)
             0 0 0 0 0 j 0 0 0 0 0 0 0 0 0 0 0 0;  % 13, +leg
             0 0 0 0 0 0 0 0 0 0 0 0 e 0 0 0 0 0;  % 14,
             0 0 0 0 0 0 0 0 0 0 0 0 0 e 0 0 0 0;  % 15, torso, +arm 
             0 0 0 0 0 0 0 0 0 0 0 0 0 0 j e 0 0;  % 16,
             0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 u 0 0;  % 17,
             0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 u e;  % 18, head
           ];
        
% correspondence between DP & KP models
kp2dp = { [18 17 16], ... %head
          [15 14 13], ... %torso
          [6 5], [6 5], [4 3], [4 3], ... %leg
          [12 11], [12 11], [10 9], [10 9], ... %arm
          [8 7], [8 7], [2 1], [2 1] ... %hand,foot
        };

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% others
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

default_width = 486;
default_height = 640;

ippal = [ 1 2 4 3 6 5 8 7 10 9 12 11 14 13 ];
for ip = 1:parts_np
  iv = parts_nvcum(ip)+1:parts_nvcum(ip+1);
  if ip==1 %head
    iv_ = [5 4 3 2 1 6];
  else
    iv_ = parts_nvcum(ippal(ip))+1:parts_nvcum(ippal(ip)+1);
    iv_ = reshape( flipdim( reshape(iv_,[2 length(iv)/2]), 1 ), [1 length(iv)] );
  end
  
  ivpal(1,iv) = iv_;
end

parts_name_up = { 'head', 'torso', ...
                  'armul', 'armur', 'armll', 'armlr', ...
                  'handl', 'handr' };
parts_name_low = { 'legul', 'legur', 'legll', 'leglr', ...
                   'footl', 'footr' };
parts_name_torso = { 'head', 'torso' };
parts_name_arm = { 'armul', 'armur', 'armll', 'armlr', ...
                   'handl', 'handr' };
parts_name_arml = { 'armul', 'armll', 'handl' };
parts_name_armr = { 'armur', 'armlr', 'handr' };
parts_name_legl = { 'legul', 'legll' };
parts_name_legr = { 'legur', 'leglr' };
links_name_up = { 'neck', ...
                  'shl', 'shr', 'elbl', 'elbr', 'wristl', 'wristr' };
links_name_low = { 'hipl', 'hipr', 'keel', 'keer', 'ankl', 'ankr' };

parts_np_up = length(parts_name_up);
parts_np_low = length(parts_name_low);
parts_np_torso = length(parts_name_torso);
parts_np_arm = length(parts_name_arm);
parts_np_arml = length(parts_name_arml);
parts_np_armr = length(parts_name_armr);
parts_np_legl = length(parts_name_legl);
parts_np_legr = length(parts_name_legr);
links_nl_up = length(links_name_up);
links_nl_low = length(links_name_low);
parts_iv_up = zeros(1,0);
parts_iv_low = zeros(1,0);
parts_iv_torso = zeros(1,0);
parts_iv_arm = zeros(1,0);
parts_iv_arml = zeros(1,0);
parts_iv_armr = zeros(1,0);
parts_iv_legl = zeros(1,0);
parts_iv_legr = zeros(1,0);
for i = 1:parts_np_up
  parts_ip_up(i) = strmatch(parts_name_up{i}, parts_name, 'exact');
  parts_iv_up = [parts_iv_up parts_nvcum(parts_ip_up(i))+1:parts_nvcum(parts_ip_up(i)+1)];
end
for i = 1:parts_np_low
  parts_ip_low(i) = strmatch(parts_name_low{i}, parts_name, 'exact');
  parts_iv_low = [parts_iv_low parts_nvcum(parts_ip_low(i))+1:parts_nvcum(parts_ip_low(i)+1)];
end
for i = 1:parts_np_torso
  parts_ip_torso(i) = strmatch(parts_name_torso{i}, parts_name, 'exact');
  parts_iv_torso = [parts_iv_torso parts_nvcum(parts_ip_torso(i))+1:parts_nvcum(parts_ip_torso(i)+1)];
end
for i = 1:parts_np_arm
  parts_ip_arm(i) = strmatch(parts_name_arm{i}, parts_name, 'exact');
  parts_iv_arm = [parts_iv_arm parts_nvcum(parts_ip_arm(i))+1:parts_nvcum(parts_ip_arm(i)+1)];
end
for i = 1:parts_np_arml
  parts_ip_arml(i) = strmatch(parts_name_arml{i}, parts_name, 'exact');
  parts_iv_arml = [parts_iv_arml parts_nvcum(parts_ip_arml(i))+1:parts_nvcum(parts_ip_arml(i)+1)];
end
for i = 1:parts_np_armr
  parts_ip_armr(i) = strmatch(parts_name_armr{i}, parts_name, 'exact');
  parts_iv_armr = [parts_iv_armr parts_nvcum(parts_ip_armr(i))+1:parts_nvcum(parts_ip_armr(i)+1)];
end
for i = 1:parts_np_legl
  parts_ip_legl(i) = strmatch(parts_name_legl{i}, parts_name, 'exact');
  parts_iv_legl = [parts_iv_legl parts_nvcum(parts_ip_legl(i))+1:parts_nvcum(parts_ip_legl(i)+1)];
end
for i = 1:parts_np_legr
  parts_ip_legr(i) = strmatch(parts_name_legr{i}, parts_name, 'exact');
  parts_iv_legr = [parts_iv_legr parts_nvcum(parts_ip_legr(i))+1:parts_nvcum(parts_ip_legr(i)+1)];
end
for i = 1:links_nl_up
  links_il_up(i) = strmatch(links_name_up{i}, links_name, 'exact');
end
for i = 1:links_nl_low
  links_il_low(i) = strmatch(links_name_up{i}, links_name, 'exact');
end
parts_iv_leg = [parts_iv_legl parts_iv_legr];
